@@ -32,20 +32,22 @@
 
 int config_no_answer = 0;
 
+static struct tagtable cmdtab[] = {
+  { "saveconfig",            TAG_SAVECONFIG},
+  { "reinit",                TAG_REINIT},
+  { "ejectcd",               TAG_EJECTCD },
+  { "sleep",                 TAG_GOTOSLEEP }};
+
 static struct tagtable tagtab[] = {
-  { "TAG_SAVECONFIG",        TAG_SAVECONFIG},
   { "TAG_AUTORESCAN",        TAG_AUTORESCAN},
   { "TAG_TIMEFORCMD",        TAG_TIMEFORCMD},
   { "TAG_VERSION",           TAG_VERSION},
   { "TAG_KEYCODE",           TAG_KEYCODE},
   { "TAG_KEYREPEAT",         TAG_KEYREPEAT},
   { "TAG_MODIFIER",          TAG_MODIFIER},
-  { "TAG_MOUSEBUTTONS",      TAG_MOUSEBUTTONS},
   { "TAG_MOUSERELX",         TAG_MOUSERELX},
   { "TAG_MOUSERELY",         TAG_MOUSERELY},
-  { "TAG_MOUSEWHEEL",        TAG_MOUSEWHEEL},
   { "TAG_SYSINFO",           TAG_SYSINFO},
-  { "TAG_REINIT",            TAG_REINIT},
   { "TAG_BACKLIGHTMAX",      TAG_BACKLIGHTMAX},
   { "TAG_BACKLIGHTLEVEL",    TAG_BACKLIGHTLEVEL},
   { "TAG_COVERSTATUS",       TAG_COVERSTATUS},
@@ -64,6 +66,7 @@ static struct tagtable tagtab[] = {
   { "TAG_TPMODEDOWNKEY",     TAG_TPMODEDOWNKEY},
   { "TAG_TPMODEDOWNMOD",     TAG_TPMODEDOWNMOD},
   { "TAG_TPMODE",            TAG_TPMODE},
+  { "TAG_NOTAPTYPING",       TAG_NOTAPTYPING},
   { "TAG_KBDMODE",           TAG_KBDMODE},
   { "TAG_BATLOG",            TAG_BATLOG},
   { "TAG_BATTERYPRESENT",    TAG_BATTERYPRESENT},
@@ -100,7 +103,13 @@ static struct tagtable tagtab[] = {
   { "TAG_FRAMEBUFFERDEVICE", TAG_FRAMEBUFFERDEVICE},
   { "TAG_BLANKFRAMEBUFFER",  TAG_BLANKFRAMEBUFFER},
   { "TAG_DIMFULLYDARK",      TAG_DIMFULLYDARK},
+  { "TAG_CRTMIRROR",         TAG_CRTMIRROR},
+  { "TAG_CRTMIRRORKEY",      TAG_CRTMIRRORKEY},
+  { "TAG_CRTMIRRORMOD",      TAG_CRTMIRRORMOD},
+  { "TAG_SOUNDSYSTEM",       TAG_SOUNDSYSTEM},
+  { "TAG_SOUNDSYSTEMCFG",    TAG_SOUNDSYSTEMCFG},
   { "TAG_MIXERDEVICE",       TAG_MIXERDEVICE},
+  { "TAG_MIXERCARD",         TAG_MIXERCARD},
   { "TAG_VOLUMEUPKEY",       TAG_VOLUMEUPKEY},
   { "TAG_VOLUMEUPMOD",       TAG_VOLUMEUPMOD},
   { "TAG_VOLUMEDOWNKEY",     TAG_VOLUMEDOWNKEY},
@@ -111,16 +120,15 @@ static struct tagtable tagtab[] = {
   { "TAG_VOLUME",            TAG_VOLUME },
   { "TAG_MUTE",              TAG_MUTE },
   { "TAG_MIXERCHANNELS",     TAG_MIXERCHANNELS },
+  { "TAG_MIXERELEMENTS",     TAG_MIXERELEMENTS },
   { "TAG_CDROMDEVICE",       TAG_CDROMDEVICE },
   { "TAG_EJECTCDKEY",        TAG_EJECTCDKEY },
   { "TAG_EJECTCDMOD",        TAG_EJECTCDMOD },
   { "TAG_EJECTCDKEYDELAY",   TAG_EJECTCDKEYDELAY },
-  { "TAG_EJECTCD",           TAG_EJECTCD },
   { "TAG_SCRIPTPMCS",        TAG_SCRIPTPMCS},
   { "TAG_SLEEPKEY",          TAG_SLEEPKEY },
   { "TAG_SLEEPMOD",          TAG_SLEEPMOD },
   { "TAG_SLEEPKEYDELAY",     TAG_SLEEPKEYDELAY },
-  { "TAG_GOTOSLEEP",         TAG_GOTOSLEEP },
   { "TAG_BWLFIRST",          TAG_BWLFIRST },
   { "TAG_BWLSECOND",         TAG_BWLSECOND },
   { "TAG_BWLLAST",           TAG_BWLLAST },
@@ -134,18 +142,6 @@ static struct tagtable tagtab[] = {
   { "TAG_NETLOADDEV",        TAG_NETLOADDEV },
   { "TAG_POLICY",            TAG_POLICY },
   
-  { "TAG_SLEEPKEYSLEEP",     TAG_SLEEPKEYSLEEP },         /* depreciated */
-  { "TAG_ONACPOLICY",        TAG_ONACPOLICY },            /* depreciated */
-  { "TAG_ONACSLEEP",         TAG_ONACSLEEP },             /* depreciated */
-  { "TAG_ONACCOVERSLEEP",    TAG_ONACCOVERSLEEP },        /* depreciated */
-  { "TAG_ONACTIMESLEEP",     TAG_ONACTIMESLEEP },         /* depreciated */
-  { "TAG_ONACTIMEDIM",       TAG_ONACTIMEDIM },           /* depreciated */
-  { "TAG_ONBATTERYPOLICY",   TAG_ONBATTERYPOLICY },       /* depreciated */
-  { "TAG_ONBATTERYSLEEP",    TAG_ONBATTERYSLEEP },        /* depreciated */
-  { "TAG_ONBATTERYCOVERSLEEP", TAG_ONBATTERYCOVERSLEEP }, /* depreciated */
-  { "TAG_ONBATTERYTIMESLEEP", TAG_ONBATTERYTIMESLEEP },   /* depreciated */
-  { "TAG_ONBATTERYTIMEDIM",  TAG_ONBATTERYTIMEDIM },      /* depreciated */
-
   { "TAG_ONAC_POLICY",       TAG_ONAC_POLICY },
   { "TAG_ONAC_TIMERACTION",  TAG_ONAC_TIMERACTION },
   { "TAG_ONAC_COVERACTION",  TAG_ONAC_COVERACTION },
@@ -189,20 +185,46 @@ void
 print_usage (char *prgname)
 {
 	printf(_("%s (version %s) - control client for pbbuttonsd.\n"), prgname, VERSION);
-	printf(_("Usage: %s [-%s] <'query'  | 'config'> <tag> [<data>]\n"), prgname, ARG_ALL);
+	printf(_("Usage:\n"));
+	printf("    %s [-%s] <command>\n", prgname, ARG_ALL);
+	printf("    %s [-%s] query  <tag>\n", prgname, ARG_ALL);
+	printf("    %s [-%s] config <tag> <data>\n\n", prgname, ARG_ALL);
 	printf (_("Options:\n"
-		"   -%c, --help               display this help and exit\n"
-		"   -%c, --version            display version information and exit\n"
-		"   -%c, --ignore             ignore config return and error values\n\n"),
+		"    -%c, --help               display this help and exit\n"
+		"    -%c, --version            display version information and exit\n"
+		"    -%c, --ignore             ignore config return and error values\n"),
 		ARG_HELP, ARG_VERSION, ARG_IGNORE);
+	print_cmds ();
 	print_tags ();
 }
 
+void
+print_cmds ()
+{
+	printf ("\n");
+	printf (_("Supported commands:\n"));
+	printf ("    ");
+	printf (_("%-14s - prints a list of clients attached\n"), "listclients");
+	printf ("    ");
+	printf (_("%-14s - save the current configuration to disk\n"), "saveconfig");
+	printf ("    ");
+	printf (_("%-14s - reinitialising the keyboard and the trackpad\n"), "reinit");
+	printf ("    ");
+	printf (_("%-14s - eject the CDROM\n"), "ejectcd");
+	printf ("    ");
+	printf (_("%-14s - suspend to RAM\n"), "sleep");
+	printf ("    ");
+	printf (_("%-14s - reconfigure server parameters\n"), "config");
+	printf ("    ");
+	printf (_("%-14s - queries the server for certain parameters\n"), "query");
+}
+
 void
 print_tags ()
 {
 	int n, z;
 
+	printf ("\n");
 	printf (_("Supported tags:\n"));
 	printf ("    ");
 	for (n=1,z=1; n <= (sizeof(tagtab) / sizeof(struct tagtable)); n++) {
@@ -238,7 +260,7 @@ evaluate_args(int argc, char *argv[], struct tagitem *taglist)
 		switch (c) {
 			case ARG_VERSION:
 				printf(_("pbbcmd, version %s"), VERSION);
-				printf(", (c) 2002 Matthias Grimm\n");
+				printf(", (c) 2002-2005 Matthias Grimm\n");
 				return CMD_FLAGS;
 			case ARG_IGNORE:
 				config_no_answer = 1;
@@ -250,11 +272,15 @@ evaluate_args(int argc, char *argv[], struct tagitem *taglist)
 		}
 	}
 
-	if (argc - optind >= 2) {
+	if (argc - optind >= 1) {
 		if ((strcasecmp (argv[optind], "query")) == 0) {
+			if (argc - optind < 2) {
+				printf (_("ERROR: Missing arguments\n"));
+				return 0;
+			}
 			for (n=optind+1; n < argc; n++) {
 				if ((tag = identifytag (argv[n])) != 0) {
-					taglist_add (taglist, tag, 0);
+					taglist_add (taglist, tag, (tag & FLG_STRING) ? 0 : -1);
 				} else {
 					printf (_("WARNING: tag %s not supported.\n"), argv[n]);
 					print_tags ();
@@ -262,25 +288,35 @@ evaluate_args(int argc, char *argv[], struct tagitem *taglist)
 				}
 			}
 			return CMD_QUERY;
+
 		} else if ((strcasecmp (argv[optind], "config")) == 0) {
-			if ((argc - optind - 1) % 2 == 1) {
+			if (argc - optind < 3) {
+				printf (_("ERROR: Missing arguments\n"));
+				return 0;
+			} else if ((argc - optind - 1) % 2 == 1) {
 				printf (_("ERROR: tag/data pairs not complete.\n"));
 				return 0;
-			} else {
-				for (n=optind+1; n < argc; n++) {
-					if ((tag = identifytag (argv[n])) != 0) {
-						if (tag & FLG_STRING)
-							taglist_add (taglist, tag, (long) argv[++n]);
-						else
-							taglist_add (taglist, tag, (long) atoi (argv[++n]));
-					} else {
-						printf (_("WARNING: tag %s not supported.\n"), argv[n]);
-						print_tags ();
-						return 0;
-					}
-				}
-				return CMD_CONFIG;
 			}
+			for (n=optind+1; n < argc; n++) {
+				if ((tag = identifytag (argv[n])) != 0) {
+					if (tag & FLG_STRING)
+						taglist_add (taglist, tag, (long) argv[++n]);
+					else
+						taglist_add (taglist, tag, (long) atoi (argv[++n]));
+				} else {
+					printf (_("WARNING: tag %s not supported.\n"), argv[n]);
+					print_tags ();
+					return 0;
+				}
+			}
+			return CMD_CONFIG;
+
+		} else if (strcasecmp (argv[optind], "listclients") == 0) {
+			return CMD_LISTCLIENTS;
+
+		} else if ((tag = identifycmd (argv[optind])) != 0) {
+			taglist_add (taglist, tag, 0);
+			return CMD_CONFIG;
 		}
 	}
 	print_usage (prgname);
@@ -316,6 +352,17 @@ identifytag (char *name)
 	return 0;
 }
 
+long
+identifycmd (char *name)
+{
+	int n;
+
+	for (n=(sizeof(cmdtab) / sizeof(struct tagtable) - 1); n >= 0; n--)
+		if (!strcasecmp (cmdtab[n].tagname, name))
+			return cmdtab[n].tag;
+	return 0;
+}
+
 char *
 identifytagname (long tag, char *defname)
 {
@@ -357,7 +404,7 @@ int main(int argc, char *argv[])
 	if ((taglist = (struct tagitem *) malloc (argc * sizeof (struct tagitem))) != NULL) {
 		taglist_init (taglist);
 		if ((cmd = evaluate_args (argc, argv, taglist)) != 0) {
-			if (ipc_init (LIBMODE_CLIENT, 0) == 0) {
+			if (ipc_init ("pbbcmd", LIBMODE_CLIENT, 0) == 0) {
 				switch (cmd) {
 				case CMD_FLAGS:  /* for command line flags -h and -v */
 					rc = 0;               /* they are no errors */
@@ -400,6 +447,37 @@ int main(int argc, char *argv[])
 							printf (_("ERROR: Unexpected answer from server, actioncode %ld.\n"), (long) pbbmsg->action);
 					}
 					break;
+				case CMD_LISTCLIENTS:
+					ipc_send(0, GETCLIENTLIST, NULL);
+					if ((rc = read_message (buffer, BUFFERLEN)) == 0) {
+						pbbmsg = (struct pbbmessage *) buffer;
+						if (pbbmsg->action == CHANGEVALUE) {
+							printf ("\n");
+							printf (_("  PID   | Name                 | Port    | Flags\n"
+							          "  ------+----------------------+---------+-----------\n"));
+	
+							tags = pbbmsg->taglist;
+							while (tags->tag != TAG_END) {
+								switch (tags->tag) {
+								case TAG_CLIENTPID:
+									printf ("  %5d |", (int) tags->data);
+									break;
+								case TAG_CLIENTNAME:
+									printf (" %-20s |", (tags->data) ? (char*) tags->data : _("<unknown>"));
+									break;
+								case TAG_CLIENTPORT:
+									printf (" %7d |", (int) tags->data);
+									break;	
+								case TAG_CLIENTFLAGS:
+									printf (" %08x\n", (int) tags->data);
+									break;
+								}
+								tags++;
+							}
+							printf ("\n");
+						}
+					}
+					break;
 				}
 				ipc_exit();
 			} else
