@@ -16,6 +16,7 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <signal.h>
 #include <string.h>
 #include "pbbinput.h"
 
@@ -26,6 +27,8 @@
 #include "init.h"
 #include "input_manager.h"
 
+extern volatile sig_atomic_t prgexit;
+
 /* --- private module data structure of the input manager --- */
 
 struct moddata_inputmanager {
@@ -38,8 +41,8 @@ struct moddata_inputmanager {
 	unsigned short lastkey;
 	struct {
 		int fd;
-		void (*handler)(int fd);
-	} input[MAXINPUTS];
+		int (*handler)(int fd);
+	} input[MAXINPUTS+1];
 	struct {
 		short timer;
 		short start;
@@ -63,12 +66,12 @@ struct moddata_inputmanager {
    the prginitab. It does module data initialisation. */
 
 int
-inputmanager_init (struct tagitem *taglist)
+inputmanager_init ()
 {
 	struct moddata_inputmanager *base = &modbase_inputmanager;
-	int x, y, n, fd;
+	int x, y, n;
 
-	for (n=0; n < MAXINPUTS; n++) {
+	for (n=0; n <= MAXINPUTS; n++) {
     	base->input[n].fd = -1;
     	base->input[n].handler = NULL;
 	}
@@ -78,12 +81,10 @@ inputmanager_init (struct tagitem *taglist)
 		base->evdevs[n].inputid = -1;
 	}
 
-	scan_for_kbdevdevs();
-	if ((fd = open("/dev/input/mice", O_RDONLY)) >= 0)  /* open mouse device */
-		register_inputhandler(fd, mouse_handler);
-
+	scanEventDevices();
+	
     /* each of TIMERCOUNT (=3) timers have to be initialised here.
-	   there is no mechanism to check is a timer slot is empty */
+	   there is no mechanism to check if a timer slot is empty */
 	base->timer[0].handler = timer10_handler;  /* timer for 10ms queue */
 	base->timer[1].start = 10;
 	base->timer[1].handler = timer100_handler;  /* timer for 100ms queue */
@@ -174,8 +175,21 @@ call_timertick ()
 
 /* --- event handler management --- */
 
+struct evdevice*
+findEventDevice (int fd)
+{
+	struct moddata_inputmanager *base = &modbase_inputmanager;
+	int n;
+
+	for (n = 0; n < EVDEVCOUNT; n++)
+		if (fd == base->evdevs[n].inputid)
+			return &base->evdevs[n];
+		
+	return NULL;
+}
+
 void
-clear_evdevice (struct evdevice *evdev)
+clearEventDevice (struct evdevice *evdev)
 {
 	if (evdev->inputid >= 0) {
 		unregister_inputhandler(evdev->inputid);
@@ -191,71 +205,43 @@ clear_evdevice (struct evdevice *evdev)
 }
 
 void
-scan_for_kbdevdevs()
+scanEventDevices ()
 {
 	struct moddata_inputmanager *base = &modbase_inputmanager;
 
-	int n, m, fd, fcnt = 0, rc;
+	int n, m, fd, fcnt = 0;
 	char filename[20];
-	unsigned long bit[NBITS(EV_MAX)];
 	unsigned short id[4];
 
 	for (n = 0, m = 0; n < EVDEVCOUNT; n++) {
 		sprintf(filename, "/dev/input/event%d", n);
 		if ((fd = open(filename, O_RDONLY)) >= 0) {
-			ioctl(fd, EVIOCGBIT(0, EV_MAX), bit);
-			if (test_bit(EV_KEY, bit) && test_bit(EV_REP, bit)) {
-				ioctl(fd, EVIOCGID, id);
-				if (id[ID_PRODUCT] != base->evdevs[m].product ||
-					id[ID_VENDOR]  != base->evdevs[m].vendor) {
-					clear_evdevice(&base->evdevs[m]);
-					base->evdevs[m].inputid = fd;
-					base->evdevs[m].product = id[ID_PRODUCT];
-					base->evdevs[m].vendor = id[ID_VENDOR];
-					register_inputhandler(fd, keyboard_handler);
+			ioctl(fd, EVIOCGID, id);
+			if (id[ID_PRODUCT] != base->evdevs[m].product ||
+				id[ID_VENDOR]  != base->evdevs[m].vendor) {
+				clearEventDevice (&base->evdevs[m]);
+				base->evdevs[m].inputid = fd;
+				base->evdevs[m].product = id[ID_PRODUCT];
+				base->evdevs[m].vendor = id[ID_VENDOR];
+				register_inputhandler(fd, event_handler);
 #ifdef DEBUG
-					printf("Inputhandler registered:\t%04x %04x         \n",id[ID_PRODUCT],id[ID_VENDOR]);
+				printf("Inputhandler registered:\t%04x %04x         \n",id[ID_PRODUCT],id[ID_VENDOR]);
 #endif
-				} else
-					close(fd);
-				m++;
 			} else
 				close(fd);
+			m++;
 		} else
 			fcnt++;  /* count number of event devices that could not be opened */
 	}
 
 	for (; m < EVDEVCOUNT; m++)    /* cleanup rest of list */
-		clear_evdevice(&base->evdevs[m]);
+		clearEventDevice (&base->evdevs[m]);
 
-	/* The keyboard actions depend on the event devices of the new
-	 * input layer of kernel 2.4 and above. It could happen that
-	 * the appropriate kernel module has not been loaded before
-	 * pbbuttonsd is lauched. In this case the following code tries
-	 * to load the module and correct the problem. It depends on
-	 * the external tools /bin/grep and /sbin/modprobe and the file
-	 * /proc/modules which have to be located as named. For security
-	 * reasons system() wasn't used.
-	 */
-	if (fcnt == EVDEVCOUNT) {  /* none of the 32 devices can be opened */
-		print_error(_("WARNING: No event devices available. Lets see what's wrong.\n"));
-		if (geteuid() != 0)
-			print_error(_("         - Running as non-root. Perhaps on that score /dev/input/event* can't be accessed.\n"));
-		else if ((rc = launch_program("/bin/grep evdev /proc/modules")) == 0)
-			print_error(_("         - Kernel module evdev.o is already loaded, must be something else.\n"));
-		else if (rc != E_CLDEXIT)
-			print_error(_("         - Failed to lauch '/bin/grep evdev /proc/modules', please check files.\n"));
-		else
-			if ((rc = launch_program("/sbin/modprobe evdev")) == 0) {
-				print_error(_("         - Kernel module evdev.o successfully loaded, that's it.\n"));
-				scan_for_kbdevdevs();
-				return;
-			} else
-				print_error(_("         - Failed to lauch '/sbin/modprobe evdev', maybe one of them is missing.\n"));
-		print_error(_("         - Can't figure out what's wrong, sorry - keyboard actions won't work.\n"));
-	}
+	if (fcnt == EVDEVCOUNT)  /* none of the 32 devices can be opened */
+		print_msg (PBB_WARN, _("No event devices available. Please check your configuration.\n"));
 }
 
+
 /* --- input handler --- */
 
 int
@@ -279,7 +265,7 @@ unregister_inputhandler (int fd)
 	struct moddata_inputmanager *base = &modbase_inputmanager;
 	int n, found = 0;
 
-	for (n = 0; n < MAXINPUTS; n++)
+	for (n=0; n <= MAXINPUTS; n++)
 		if (found) {
 			base->input[n-1].fd = base->input[n].fd;
 			base->input[n-1].handler = base->input[n].handler;
@@ -287,7 +273,7 @@ unregister_inputhandler (int fd)
 			found = 1;
 	if (found)
 		base->input[n].fd = -1;
-}
+}	
 
 int
 create_fdset (fd_set *watchset)
@@ -309,13 +295,30 @@ void
 call_inputhandler(fd_set *inset)
 {
 	struct moddata_inputmanager *base = &modbase_inputmanager;
-	int n;
+	struct evdevice *evdev;
+	int n = 0, rescan = 0;
 
-	for (n=0; n < MAXINPUTS; n++) {
-		if (base->input[n].fd == -1) break;
-		if (FD_ISSET(base->input[n].fd, inset))
-			base->input[n].handler (base->input[n].fd);
+	while (n < MAXINPUTS && base->input[n].fd != -1) {
+		if (FD_ISSET(base->input[n].fd, inset)) {
+			if ((base->input[n].handler (base->input[n].fd)) == -1) {
+		    	if ((evdev = findEventDevice (base->input[n].fd))) {
+					clearEventDevice (evdev);
+					rescan = 1;
+				} else {
+					print_msg (PBB_ERR,
+							_("Can't read filehandle %d anymore ... exiting!\n"),
+							base->input[n].fd);
+					prgexit = 1;
+					return;
+				}
+			} else
+				n++;
+		} else
+			n++;
 	}
+	
+	if (rescan)     /* one or more filehandles have failed and must be closed */
+		scanEventDevices ();  /* so rescan the event devices to fill the gap */
 }
 
 /* --- queue managers --- */
@@ -364,7 +367,6 @@ process_queue (int queueid, struct tagitem *taglist)
 	}
 }
 
-
 /* --- input handlers --- */
 
 void
@@ -400,40 +402,46 @@ ipc_handler ()
 	}
 }
 
-void
-keyboard_handler (int fd)
+int
+event_handler (int fd)
 {
-  struct input_event inp;
-  struct tagitem taglist[] = {{ TAG_KEYCODE, 0 },
-                              { TAG_KEYREPEAT, 0 },
-                              { TAG_MODIFIER, 0 },
-                              { TAG_END, 0 }};
+	struct input_event inp;
+	struct tagitem taglist[10];
 
-  if (read(fd, &inp, sizeof(inp)) == sizeof(inp)) {
-    taglist[0].data = (long) inp.code;
-    taglist[1].data = (long) inp.value;
-    taglist[2].data = (long) track_modifier (inp.code, inp.value);
-    process_queue (KBDQUEUE, taglist);
-  }
-}
-
-void
-mouse_handler (int fd)
-{
-  signed char buffer[] = {0, 0, 0 ,0};
-  struct tagitem taglist[] = {{ TAG_MOUSEBUTTONS, 0 },
-                              { TAG_MOUSERELX, 0 },
-                              { TAG_MOUSERELY, 0 },
-                              { TAG_MOUSEWHEEL, 0 },
-                              { TAG_END, 0 }};
-  int count, n=0;
-
-  if ((count = read(fd, buffer, sizeof(buffer))) > 0) {
-    for (n=0; n < count; n++)
-      taglist[n].data = (long) buffer[n];
-    taglist[n].tag = TAG_END;
-    process_queue (MOUSEQUEUE, taglist);
-  }
+	taglist_init(taglist);
+	
+	if (read(fd, &inp, sizeof(inp)) == sizeof(inp)) {
+		switch (inp.type) {
+		case EV_KEY:
+			/* the left mouse button must not be sent because this will
+			 * break NoTapTyping. The Trackpad will emulate a left mouse
+			 * button which causes that 'tapping' will be disabled. The
+			 * event system received a 'mouse button down' but a 'mouse
+			 * button up' will never be seen. This leads to a weird
+			 * trackpad behaviour.
+			 */
+			if (inp.code == BTN_LEFT)
+				break;
+			
+			/* all other key events will be forwarded */
+			taglist_add (taglist, TAG_KEYCODE,  (long) inp.code);
+			taglist_add (taglist, TAG_KEYREPEAT,(long) inp.value);
+			taglist_add (taglist, TAG_MODIFIER, (long) track_modifier (inp.code, inp.value));
+			process_queue (KBDQUEUE, taglist);
+			break;
+		case EV_REL:
+			if (inp.code)
+				taglist_add (taglist, TAG_MOUSERELY, (long) inp.value);
+			else
+				taglist_add (taglist, TAG_MOUSERELX, (long) inp.value);
+    		process_queue (MOUSEQUEUE, taglist);
+			break;
+		default:
+			break; /* catch unknown event types */
+		}
+		return 0;
+	} else
+		return -1; /* read has failed */
 }
 
 void
