@@ -28,6 +28,7 @@
 #include "pbbinput.h"
 #include "gettext_macros.h"
 #include "input_manager.h"
+#include "config_manager.h"
 #include "module_cdrom.h"
 #include "support.h"
 #include "debug.h"
@@ -42,11 +43,11 @@ struct moddata_cdrom {
 } modbase_cdrom;
 
 int
-cdrom_init (struct tagitem *taglist)
+cdrom_init ()
 {
 	struct moddata_cdrom *base = &modbase_cdrom;
 	static char devbuffer_cdrom[STDBUFFERLEN];
-	int rc;
+	int sid;
 
 	sprintf(devbuffer_cdrom, DEFAULT_CDROM);
 
@@ -55,16 +56,46 @@ cdrom_init (struct tagitem *taglist)
 	base->modejectcd	= MOD_NONE;
 	base->keydelaytime	= 0;
 
-	if ((rc = cdrom_handle_tags (MODE_CONFIG, taglist))) /* interprete configfile */
-		return rc;                 /* critcal error occured, abort program */
+	if ((sid = registerCfgSection ("MODULE CDROM")) == 0) {
+		print_msg (PBB_ERR, _("Can't register configuration section %s, out of memory."), "MODULE CDROM");
+		return E_NOMEM;
+	} else {
+		registerCfgOptionString (sid, "dev_CDROM", TAG_CDROMDEVICE, 0,
+				NULL);
+		registerCfgOptionKey (sid, "EjectCDKey", TAG_EJECTCDKEY, TAG_EJECTCDMOD, 0,
+				NULL);
+		registerCfgOptionInt (sid, "EjectCDKeyDelay", TAG_EJECTCDKEYDELAY, 0,
+				NULL);
+	}
 
+	register_function (QUERYQUEUE, cdrom_query);
+	register_function (CONFIGQUEUE, cdrom_configure);
+	return 0;
+}
+
+int
+cdrom_open (struct tagitem *taglist)
+{
+	struct moddata_cdrom *base = &modbase_cdrom;
+	char *devname;
+	int rc;
+
+	devname = (char *) tagfind (taglist, TAG_CDROMDEVICE, (long) DEFAULT_CDROM);
+	if ((rc = copy_path (devname, base->cdromdev, TYPE_BLKDEV, CPFLG_MAYBEMISSED))) {
+		print_msg (PBB_ERR, _("Please check your CDROM settings in configuration file.\n"));
+		return rc;
+	}
+		
 #if defined(DEBUG) && CDROMPERF
 	cdrom_testperformance ();
 #endif	
-	
 	register_function (KBDQUEUE, cdrom_keyboard);
-	register_function (QUERYQUEUE, cdrom_query);
-	register_function (CONFIGQUEUE, cdrom_configure);
+	return 0;
+}
+
+int
+cdrom_close ()
+{
 	return 0;
 }
 
@@ -107,11 +138,11 @@ cdrom_configure (struct tagitem *taglist)
 	cdrom_handle_tags(MODE_CONFIG, taglist);
 }
 
-int
+void
 cdrom_handle_tags (int cfgure, struct tagitem *taglist)
 {
 	struct moddata_cdrom *base = &modbase_cdrom;
-	int err, rc = 0;
+	int err;
 
 	while (taglist->tag != TAG_END) {
 		switch (taglist->tag) {
@@ -126,7 +157,7 @@ cdrom_handle_tags (int cfgure, struct tagitem *taglist)
 		case TAG_CDROMDEVICE:
 			if (cfgure) {
 				if ((err = copy_path ((char *) taglist->data, base->cdromdev, TYPE_BLKDEV, CPFLG_MAYBEMISSED)))
-					rc = tagerror (taglist, err);
+					tagerror (taglist, err);
 			} else
 				taglist->data = (long) base->cdromdev;
 			break;
@@ -145,27 +176,108 @@ cdrom_handle_tags (int cfgure, struct tagitem *taglist)
 		}
 		taglist++;
 	}
-	return rc;
 }
 
+/* This function checks is a given name could be found in
+ * a list of names. It returns a boolean walue:
+ *  1 = list contains name
+ *  0 = name could not be found in list
+ */
+int
+cdrom_cmpDevName(struct listhead *list, char *name)
+{
+	struct listnode *it = list->first;
+	
+	while ((it = it->next))
+		if (strcmp (name, (char*) getNodeData(it->pred)) == 0)
+			return 1;
+	return 0;
+}
+
+/* This function adds 'name' to a double linked list. The
+ * memory for the node and the name is dynamically allocated.
+ * The name is copied to the newly allocated memory so that
+ * the caller must no longer take care about it.
+ * The function returns -1 on error (out of memory).
+ */
+int
+cdrom_addDevName(struct listhead *list, char *name)
+{	
+	struct listnode *node;
+	
+	if((node = newListNode (strlen(name)+1))) {
+		strcpy ((char*)getNodeData(node), name);
+		appendListNode (list, node);
+		return 0;
+	}
+	return -1;  /* out of memory */
+}
+
+/* This function gets the mountpoint of a given device. The source
+ * of mountpoints is the file /proc/mounts. The function compares
+ * if the device is mounted and where. Symlinks will be resolved
+ * so that if a link to the device was mounted the mountpoint
+ * would be found too.
+ *
+ * symlink chain: /dev/cdrom -> /dev/cd1 -> /dev/hdc
+ * cdrom mounted: /dev/cd1
+ *
+ * The mountpount will be found either /dev/cdrom or /dev/hdc is
+ * given.
+ */
 char *
 cdrom_getmountpoint(char *device)
 {
-	char *mountpoint = NULL, *token;
 	static char buffer[100];
+	struct listhead devs;
+	char linkbuf[50], *mountpoint = NULL, *token;
 	FILE *fd;
+	int n;
 
+	initListHeader (&devs);
+
+	strcpy (buffer, device);
+	do {
+		if (cdrom_addDevName (&devs, buffer))
+			return NULL; /* out of memory */
+		if ((n = readlink(buffer, linkbuf, sizeof(linkbuf)-1)) >= 0) {
+			linkbuf[n] = '\0';  /* readlink() doesn't terminate buffer */
+			if (linkbuf[0] == '/')  /* link is absolute */
+				strcpy (buffer, linkbuf);
+			else if ((token = strrchr (buffer, '/'))) {
+				*(++token) = 0; /* terminate path behind last '/' */
+				strcat (buffer, linkbuf);
+			} else
+				break;
+		}
+	} while (n != -1);
+	
 	if ((fd = fopen ("/proc/mounts","r"))) {
-		while (fgets (buffer, sizeof (buffer), fd))
+		while (fgets (buffer, sizeof (buffer), fd)) {
+			if (buffer[0] != '/') continue;  /* check only real devices */
+#if defined(DEBUG) && CDROM
+			printf("mount  = %s", buffer);
+#endif
 			if ((token = strtok (buffer," \n"))) {
-				if (!strcmp (device, token)) {
-					mountpoint = strtok(0," \n");
-					break;
-				} else
-					strtok (0,"\n");
+				do {
+#if defined(DEBUG) && CDROM
+			        printf("  dev  = %s\n", token);
+#endif
+					if ((cdrom_cmpDevName (&devs, token))) {
+						mountpoint = strtok(0," \n");
+						goto out;
+					}	
+					if ((n = readlink(token, linkbuf, sizeof(linkbuf)-1)) >= 0)
+						buffer[n] = '\0';  /* readlink() doesn't terminate buffer */
+					token = linkbuf;
+				} while (n != -1);
 			}
+		}
+out:
 		fclose(fd);
 	}
+	
+	freeList (&devs);
 	return mountpoint;
 }
 
@@ -195,7 +307,7 @@ cdrom_eject()
 	printf ("mp=%s, err=%d, %s (%d)\n", mp, err, strerror(errno), errno);
 #endif
 
-	if (mp == NULL || err == 0 || errno == EINVAL)
+	if (mp == NULL || err == 0 || errno == EINVAL) {
 		if ((fd = open(base->cdromdev, O_RDONLY | O_NONBLOCK | O_NDELAY)) >= 0) {
 			
 			err = ioctl (fd, CDROM_DRIVE_STATUS);
@@ -226,7 +338,9 @@ cdrom_eject()
 				rc = 1;
 			}
 			close (fd);
-		}
+		} else
+			print_msg (PBB_WARN, _("Can't open %s. Eject CDROM won't work.\n"), base->cdromdev);
+	}
 	return rc;
 }
 
