@@ -36,15 +36,14 @@ static struct tagtable tagtab[] = {
   { "TAG_USERALLOWED",       TAG_USERALLOWED},
   { "TAG_AUTORESCAN",        TAG_AUTORESCAN},
   { "TAG_TIMEFORCMD",        TAG_TIMEFORCMD},
+  { "TAG_CFGNAME",           TAG_CFGNAME},
   { "TAG_VERSION",           TAG_VERSION},
   { "TAG_REINIT",            TAG_REINIT}, 
   { "TAG_KEYCODE",           TAG_KEYCODE},
   { "TAG_KEYREPEAT",         TAG_KEYREPEAT},
   { "TAG_MODIFIER",          TAG_MODIFIER},
-  { "TAG_MOUSEBUTTONS",      TAG_MOUSEBUTTONS},
   { "TAG_MOUSERELX",         TAG_MOUSERELX},
   { "TAG_MOUSERELY",         TAG_MOUSERELY},
-  { "TAG_MOUSEWHEEL",        TAG_MOUSEWHEEL},
   { "TAG_SYSINFO",           TAG_SYSINFO},
   { "TAG_BACKLIGHTMAX",      TAG_BACKLIGHTMAX},
   { "TAG_BACKLIGHTLEVEL",    TAG_BACKLIGHTLEVEL},
@@ -76,6 +75,7 @@ static struct tagtable tagtab[] = {
   { "TAG_TPMODEDOWNKEY",     TAG_TPMODEDOWNKEY},
   { "TAG_TPMODEDOWNMOD",     TAG_TPMODEDOWNMOD},
   { "TAG_TPMODE",            TAG_TPMODE},
+  { "TAG_NOTAPTYPING",       TAG_NOTAPTYPING},
   { "TAG_KBDMODE",           TAG_KBDMODE},
   { "TAG_REPLACEPMUD",       TAG_REPLACEPMUD},
   { "TAG_KEYBLIGHTLEVEL",    TAG_KEYBLIGHTLEVEL },
@@ -108,8 +108,14 @@ static struct tagtable tagtab[] = {
   { "TAG_BRIGHTNESSOP",      TAG_BRIGHTNESSOP},
   { "TAG_FRAMEBUFFERDEVICE", TAG_FRAMEBUFFERDEVICE},
   { "TAG_BLANKFRAMEBUFFER",  TAG_BLANKFRAMEBUFFER},
+  { "TAG_CRTMIRROR",         TAG_CRTMIRROR},
+  { "TAG_CRTMIRRORKEY",      TAG_CRTMIRRORKEY},
+  { "TAG_CRTMIRRORMOD",      TAG_CRTMIRRORMOD},
   { "TAG_DIMFULLYDARK",      TAG_DIMFULLYDARK},
+  { "TAG_SOUNDSYSTEM",       TAG_SOUNDSYSTEM},
+  { "TAG_SOUNDSYSTEMCFG",    TAG_SOUNDSYSTEMCFG},
   { "TAG_MIXERDEVICE",       TAG_MIXERDEVICE},
+  { "TAG_MIXERCARD",         TAG_MIXERCARD},
   { "TAG_VOLUMEUPKEY",       TAG_VOLUMEUPKEY},
   { "TAG_VOLUMEUPMOD",       TAG_VOLUMEUPMOD},
   { "TAG_VOLUMEDOWNKEY",     TAG_VOLUMEDOWNKEY},
@@ -120,6 +126,7 @@ static struct tagtable tagtab[] = {
   { "TAG_VOLUME",            TAG_VOLUME },
   { "TAG_MUTE",              TAG_MUTE },
   { "TAG_MIXERCHANNELS",     TAG_MIXERCHANNELS },
+  { "TAG_MIXERELEMENTS",     TAG_MIXERELEMENTS },
   { "TAG_CDROMDEVICE",       TAG_CDROMDEVICE },
   { "TAG_EJECTCDKEY",        TAG_EJECTCDKEY },
   { "TAG_EJECTCDMOD",        TAG_EJECTCDMOD },
@@ -130,7 +137,6 @@ static struct tagtable tagtab[] = {
   { "TAG_SLEEPKEY",          TAG_SLEEPKEY },
   { "TAG_SLEEPMOD",          TAG_SLEEPMOD },
   { "TAG_SLEEPKEYDELAY",     TAG_SLEEPKEYDELAY },
-  { "TAG_SLEEPKEYSLEEP",     TAG_SLEEPKEYSLEEP },
   { "TAG_GOTOSLEEP",         TAG_GOTOSLEEP },
   { "TAG_BWLFIRST",          TAG_BWLFIRST },
   { "TAG_BWLSECOND",         TAG_BWLSECOND },
@@ -159,26 +165,6 @@ static struct tagtable tagtab[] = {
   { "TAG_ONBATT_TIMEDIM",    TAG_ONBATT_TIMEDIM },
   { "TAG_EMERGENCYACTION",   TAG_EMERGENCYACTION },
   { "TAG_IDENTITY",          TAG_IDENTITY },
-
-  { "TAG_ONACPOLICY",        TAG_ONACPOLICY         | FLG_DEPRECIATED },
-  { "TAG_ONACSLEEP",         TAG_ONACSLEEP          | FLG_DEPRECIATED },
-  { "TAG_ONACCOVERSLEEP",    TAG_ONACCOVERSLEEP     | FLG_DEPRECIATED },
-  { "TAG_ONACTIMESLEEP",     TAG_ONACTIMESLEEP      | FLG_DEPRECIATED },
-  { "TAG_ONACTIMEDIM",       TAG_ONACTIMEDIM        | FLG_DEPRECIATED },
-  { "TAG_ONBATTERYPOLICY",   TAG_ONBATTERYPOLICY    | FLG_DEPRECIATED },
-  { "TAG_ONBATTERYSLEEP",    TAG_ONBATTERYSLEEP     | FLG_DEPRECIATED },
-  { "TAG_ONBATTERYCOVERSLEEP",TAG_ONBATTERYCOVERSLEEP | FLG_DEPRECIATED },
-  { "TAG_ONBATTERYTIMESLEEP",TAG_ONBATTERYTIMESLEEP | FLG_DEPRECIATED },
-  { "TAG_ONBATTERYTIMEDIM",  TAG_ONBATTERYTIMEDIM   | FLG_DEPRECIATED },
-  { "TAG_ONBATTERYTIMEHDOFF",TAG_ONBATTERYTIMEHDOFF | FLG_DEPRECIATED },
-  { "TAG_ONBATTERYDIM",      TAG_ONBATTERYDIM       | FLG_DEPRECIATED },
-  { "TAG_ONBATTERYBLANK",    TAG_ONBATTERYBLANK     | FLG_DEPRECIATED },
-  { "TAG_ONACTIMEHDOFF",     TAG_ONACTIMEHDOFF      | FLG_DEPRECIATED },
-  { "TAG_ONACDIM",           TAG_ONACDIM            | FLG_DEPRECIATED },
-  { "TAG_ONACBLANK",         TAG_ONACBLANK          | FLG_DEPRECIATED },
-  { "TAG_SCRIPTBATCRITICAL", TAG_SCRIPTBATCRITICAL  | FLG_DEPRECIATED },
-  { "TAG_SCRIPTPROFCHANGED", TAG_SCRIPTPROFCHANGED  | FLG_DEPRECIATED }, 
-  { "TAG_SCRIPTHDSETUP",     TAG_SCRIPTHDSETUP      | FLG_DEPRECIATED },
   { NULL, 0 }};
 
   static struct tagtable errortab[] = {
@@ -210,7 +196,13 @@ struct moddata_peep {
 } modbase_peep;
 
 int
-peep_init (struct tagitem *taglist)
+peep_init ()
+{
+	return peep_open (NULL);
+}
+
+int
+peep_open (struct tagitem *taglist)
 {
 	struct moddata_peep *base = &modbase_peep;
 
@@ -258,14 +250,9 @@ peep_keyboard (struct tagitem *taglist)
 void
 peep_mouse (struct tagitem *taglist)
 {
-	int buttons = (int) tagfind (taglist, TAG_MOUSEBUTTONS, 0);
 	int rel_x = (int) tagfind (taglist, TAG_MOUSERELX, 0);
 	int rel_y = (int) tagfind (taglist, TAG_MOUSERELY, 0);
-	int wheel = (int) tagfind (taglist, TAG_MOUSEWHEEL, -1);
-	if (wheel == -1)
-		printf("Mouse: buttons %x, rel-X %d, rel-Y %d\n", buttons, rel_x, rel_y);
-	else
-		printf("Mouse: buttons %x, rel-X %d, rel-Y %d, wheel %d\n", buttons, rel_x, rel_y, wheel);
+	printf("Mouse: Rel-X %3d, Rel-Y %3d\n", rel_x, rel_y);
 }
 #endif
 
@@ -349,7 +336,9 @@ peep_taglist (char *source, struct tagitem *taglist)
 					mark = ' ';
 
 				printf ("%-10s: %c %-25s: ", source, mark, tt->tagname);
-				if (tt->tag == TAG_SKIP) {
+				if (taglist->data == -1) {
+					printf ("tag not filled");	
+				} else if (tt->tag == TAG_SKIP) {
 					tt2 = findttentry (tagtab,  taglist->data);
 					if (tt2 != NULL)
 						printf ("%s", tt2->tagname);
