@@ -34,6 +34,7 @@
 
 #include "gettext_macros.h"
 #include "input_manager.h"
+#include "config_manager.h"
 #include "module_pmac.h"
 #include "support.h"
 #ifdef WITH_PMUD
@@ -41,6 +42,8 @@
 #endif
 #include "debug.h"
 
+#define TRACKPAD_PAUSETIME	6	/* time in 1/100 of a second */
+
 struct moddata_pmac {
 	char *pmudev;    /* pathname of the PMU device */
 	int fd_pmu;      /* filehandle of the PMU device */
@@ -49,6 +52,7 @@ struct moddata_pmac {
 
 	int version;     /* PMU version */
 	char *identity;  /* Identity string of this laptop */
+	int machine;     /* OpenFirmware machine code */
 	struct modflags_pmac flags;
 	int flagschanged;        /* PMU flags that have changed recently */
 	int oharevolbutton;      /* level of volume button on OHARE PBs scaled to 0..100 */
@@ -67,6 +71,7 @@ struct moddata_pmac {
 	unsigned short keytpmodedn;
 	unsigned short modtpmodedn;
 	int trackpad_mode;
+	int trackpad_pause;            /* trackpad switch-off timeout */
 	int keyboard_mode;             /* keyboard mode for recent keyboards (0xC[0..4]) */
 	int batlog_mode;
 	int batlog_cycle;
@@ -78,46 +83,90 @@ struct moddata_pmac {
 } modbase_pmac;
 
 int
-pmac_init (struct tagitem *taglist)
+pmac_init ()
 {
 	struct moddata_pmac *base = &modbase_pmac;
 	static char devbuffer_pmu[STDBUFFERLEN];
 	static char devbuffer_adb[STDBUFFERLEN];
 	static char identitystring[STDBUFFERLEN];
+	int sid;
+	
+	bzero (base, sizeof (struct moddata_pmac));
+	bzero (devbuffer_pmu, STDBUFFERLEN);
+	bzero (devbuffer_adb, STDBUFFERLEN);
+	bzero (identitystring, STDBUFFERLEN);
+	base->pmudev   = devbuffer_pmu;
+	base->adbdev   = devbuffer_adb;
+	base->identity = identitystring;
+
+	if ((sid = registerCfgSection ("MODULE PMAC")) == 0) {
+		print_msg (PBB_ERR, _("Can't register configuration section %s, out of memory."), "MODULE PMAC");
+		return E_NOMEM;
+	} else {
+		registerCfgOptionString (sid, "dev_PMU", TAG_PMUDEVICE, 0,
+				NULL);
+		registerCfgOptionString (sid, "dev_ADB", TAG_ADBDEVICE, 0,
+				NULL);
+		registerCfgOptionKey (sid, "TPModeUpKey", TAG_TPMODEUPKEY, TAG_TPMODEUPMOD, 0,
+				NULL);
+		registerCfgOptionKey (sid, "TPModeDownKey", TAG_TPMODEDOWNKEY, TAG_TPMODEDOWNMOD, 0,
+				NULL);
+		registerCfgOptionList (sid, "TPMode", TAG_TPMODE, 0,
+				N_("'"TRACKPAD_NOTAP_NAME"', '"TRACKPAD_TAP_NAME"', '"TRACKPAD_DRAG_NAME"' or '"TRACKPAD_LOCK_NAME"'"),
+				TRACKPAD_NOTAP_NAME, TRACKPAD_TAP_NAME, TRACKPAD_DRAG_NAME, TRACKPAD_LOCK_NAME, NULL);
+		registerCfgOptionList (sid, "KBDMode", TAG_KBDMODE, 0,
+				N_("'"KEYBOARD_FNBACK_NAME"' or '"KEYBOARD_FNTOP_NAME"'"),
+				KEYBOARD_FNBACK_NAME, KEYBOARD_FNTOP_NAME, NULL);
+		registerCfgOptionList (sid, "Batlog", TAG_BATLOG, 0,
+				N_("'"BATLOG_NONE_NAME"', '"BATLOG_CYCLE_NAME"' or '"BATLOG_LOG_NAME"'"),
+				BATLOG_NONE_NAME, BATLOG_CYCLE_NAME, BATLOG_LOG_NAME, NULL);
+		registerCfgOptionBool (sid, "NoTapTyping", TAG_NOTAPTYPING, 0,
+				N_("switch trackpad to 'notap-mode' while typing"));
+	}
+
+	register_function (QUERYQUEUE, pmac_query);
+	register_function (CONFIGQUEUE, pmac_configure);
+	return 0;
+}
+
+
+int
+pmac_open (struct tagitem *taglist)
+{
+	struct moddata_pmac *base = &modbase_pmac;
 	char *devname;
 	int val, rc;
 
-	base->pmudev		= devbuffer_pmu;
-	base->adbdev		= devbuffer_adb;
-	base->fd_pmu		= -1;
-	base->fd_adb		= -1;
-	base->flags.coveropen   = 1;           /* lid is open */
+	base->fd_pmu		    = -1;
+	base->fd_adb		     = -1;
+	base->flags.coveropen     = 1;         /* lid is open */
 	base->flags.sleepsupported = 0;        /* sleep not supported by default */
-	base->flags.set_tpmode  = 0;           /*  \  */
-	base->flags.set_kbdmode = 0;           /*    reguest flags for delayed actions */
-	base->flags.goto_sleep  = 0;           /*  /  */
+	base->flags.set_kbdmode  = 0;          /*    request flags for delayed actions */
+	base->flags.goto_sleep   = 0;          /*  /  */
+	base->flags.notap_typing = 0;
 
 	base->keytpmodeup	= KEY_BRIGHTNESSUP;
 	base->modtpmodeup	= MOD_ALT;
 	base->keytpmodedn	= KEY_BRIGHTNESSDOWN;
 	base->modtpmodedn	= MOD_ALT;
 	base->trackpad_mode	= -1;
+	base->trackpad_pause = 0;
 	base->keyboard_mode	= -1;
 	base->batlog_mode	= BATLOG_NONE;  /* no batlog */
 	base->batlog_cycle	= -1;         /* cycle invalid */
-	base->i2c_bus_lmu       = -1;  /* not available by default */
-	base->timeleft          = -1;
+	base->i2c_bus_lmu   = -1;       /* not available by default */
+	base->timeleft      = -1;
 #if defined(DEBUG) && SIMUAMBIENT
-	base->ambient           = 0;
+	base->ambient       = 0;
 #endif
 #ifdef WITH_PMUD
-	print_error(_("INFO: pmud support compiled in and active.\n"));
+	print_msg (PBB_INFO, _("pmud support compiled in and active.\n"));
 #endif
 
 	devname = (char *) tagfind (taglist, TAG_PMUDEVICE, (long) DEFAULT_PMU);
 	if ((rc = copy_path (devname, base->pmudev, TYPE_CHARDEV, CPFLG_NONE)) == 0) {
 		if ((base->fd_pmu = open(base->pmudev, O_RDWR)) < 0) {
-			print_error(_("ERROR: Can't open PMU device %s: %s\n"), devname, strerror(errno));
+			print_msg (PBB_ERR, _("Can't open PMU device %s: %s\n"), devname, strerror(errno));
 			return E_OPEN;
 		}
 	} else return rc;
@@ -125,31 +174,37 @@ pmac_init (struct tagitem *taglist)
 	devname = (char *) tagfind (taglist, TAG_ADBDEVICE, (long) DEFAULT_ADB);
 	if ((rc = copy_path (devname, base->adbdev, TYPE_CHARDEV, CPFLG_NONE)) == 0) {
 		if ((base->fd_adb = open(base->adbdev, O_RDWR)) < 0) {
-			print_error(_("ERROR: Can't open ADB device %s: %s\n"), devname, strerror(errno));
+			print_msg (PBB_ERR, _("Can't open ADB device %s: %s\n"), devname, strerror(errno));
 			return E_OPEN;
 		}
 	} else return rc;
 	
 	if ((val = lmu_find_bus()) >= 0) {
 		base->i2c_bus_lmu = val;
-		print_error(_("INFO: LMU found, ambient light sensor and keyboard illumination active.\n"));
+		print_msg (PBB_INFO, _("LMU found, ambient light sensor and keyboard illumination active.\n"));
 	} else if (val == (-E_PERM))
-		print_error(_("WARNING: Can't access i2c devices. Permission denied.\n"));
+		print_msg (PBB_WARN, _("Can't access i2c devices. Permission denied.\n"));
 #ifdef DEBUG
 	else
-		print_error("DBG: No LMU present in this ancient PowerBook.\n");
+		print_msg (PBB_INFO, "DBG: No LMU present in this ancient PowerBook.\n");
 #endif
 
 	base->version  = get_pmu_version(base->fd_adb);
-	pmac_identify(identitystring, base->version);
-	base->identity = identitystring;
+	pmac_identify(base->identity, base->version);
 	ioctl(base->fd_pmu, PMU_IOC_CAN_SLEEP, &val);
 	base->flags.sleepsupported = val == 1 ? 1 : 0; /* check if sleep is supported on this system */
 
 	pmac_update_flags ();
 	pmac_update_batteryinfo ();
 	base->flagschanged = 0;
+	base->machine = pmac_getmachine ();
 
+#ifdef DEBUG
+	print_msg (PBB_INFO, "DBG: Machine:  %0x\n", base->machine);	
+    print_msg (PBB_INFO, "DBG: Keyboard: %s\n",  haveADBKeyboard(base->machine) ? "ADB" : "USB");
+    print_msg (PBB_INFO, "DBG: Trackpad: %s\n",  haveADBTrackpad(base->machine) ? "ADB" : "USB");
+#endif
+	
 	/* Every frame buffer driver could register a backlight controller,
 	 * but only the right one is accepted. Verification is done by checking
 	 * OpenFirmware's property "backlight-control" in register_backlight_controller().
@@ -164,31 +219,28 @@ pmac_init (struct tagitem *taglist)
 		pmac_set_lcdbacklight (val);  /* restore LCD backlight level */
 	}
 		
-	if ((rc = pmac_handle_tags (MODE_CONFIG, taglist))) /* interprete configfile */
-		return rc;         /* critcal error occured, abort program */
-
-	if (base->trackpad_mode == -1)							/* tpmode defined? */
-		base->trackpad_mode = trackpad_get_config(base->fd_adb);	/* no, then get current mode */
+	base->trackpad_mode = trackpad_get_config(base->fd_adb); /* get trackpad mode */
 
 	if (base->version != OHARE_PMU) {
 		ioctl(base->fd_pmu, PMU_IOC_GRAB_BACKLIGHT, 0);
-		if (base->keyboard_mode == -1)
-			base->keyboard_mode = keyboard_get_config(base->fd_adb);	/* get keyboard config*/
+		base->keyboard_mode = keyboard_get_config(base->fd_adb);	/* get keyboard config*/
 	}
 
 	if ((register_inputhandler (dup(base->fd_pmu), pmac_pmu_handler)) < 0)
-		print_error(_("WARNING: Can't install PMU input handler. Some functionality may be missing.\n"));
+		print_msg (PBB_WARN, _("Can't install PMU input handler. Some functionality may be missing.\n"));
 
-	register_function (KBDQUEUE, pmac_keyboard);
+	/* register keyboard handler for trackpad control only if we have an ADB keyboard */
+	if (haveADBTrackpad ())
+		register_function (KBDQUEUE, pmac_keyboard);
+	
+	register_function (MOUSEQUEUE, pmac_mouse);
 	register_function (T100QUEUE, pmac_timer);
-	register_function (QUERYQUEUE, pmac_query);
-	register_function (CONFIGQUEUE, pmac_configure);
 	register_function (SECUREQUEUE, pmac_secure);
 	return 0;
 }
 
 int
-pmac_exit ()
+pmac_close ()
 {
 	struct moddata_pmac *base = &modbase_pmac;
 
@@ -201,6 +253,12 @@ pmac_exit ()
 	return 0;
 }
 
+int
+pmac_exit ()
+{
+	return 0;
+}
+
 void
 pmac_keyboard (struct tagitem *taglist)
 {
@@ -211,7 +269,22 @@ pmac_keyboard (struct tagitem *taglist)
 	value = (int) tagfind (taglist, TAG_KEYREPEAT, 0);
 	mod = (int) tagfind (taglist, TAG_MODIFIER, 0);
 
+	if (value == 1 || value == 2) {
+		/* trackpad tapping control */
+		if (!isModifier (code, mod) && base->flags.notap_typing) {
+			if (base->trackpad_pause == 0) {
+				trackpad_set_config(base->fd_adb, TRACKPAD_NOTAP);
+#if defined(DEBUG) && TRACKPAD
+				printf ("Trackpad tapping disabled\r");
+				fflush (stdout);
+#endif	
+			}
+			base->trackpad_pause = TRACKPAD_PAUSETIME;
+		}
+	}
+	
 	if (value == 1) {
+		/* trackpad mode keys */
 		if ((code == base->keytpmodeup) && (mod == base->modtpmodeup)) {
 			if (++base->trackpad_mode > TRACKPAD_LAST)
 				base->trackpad_mode = 0;
@@ -220,22 +293,35 @@ pmac_keyboard (struct tagitem *taglist)
 				base->trackpad_mode = TRACKPAD_LAST;
 		} else return;
 
-		base->flags.set_tpmode = 1;     /* request a secure function call */
+		trackpad_set_config(base->fd_adb, base->trackpad_mode);
 		singletag_to_clients (CHANGEVALUE, TAG_TPMODE, base->trackpad_mode);
 	}
 }
 
-/* This function is called when the user is idle. That means no key is pressed */
+void
+pmac_mouse (struct tagitem *taglist)
+{
+	struct moddata_pmac *base = &modbase_pmac;
 
+	/* trackpad tapping control */
+	if (base->trackpad_pause) {
+		base->trackpad_pause = 0;
+		trackpad_set_config(base->fd_adb, base->trackpad_mode);
+#if defined(DEBUG) && TRACKPAD
+		printf ("Trackpad tapping enabled \r");
+		fflush (stdout);
+#endif	
+	}
+}
+
+/* This function is called when the user is idle. That means no key is pressed
+ */
 void
 pmac_secure (struct tagitem *taglist)
 {
 	struct moddata_pmac *base = &modbase_pmac;
 	
-	if (base->flags.set_tpmode == 1) {
-		base->flags.set_tpmode = 0;
-		trackpad_set_config(base->fd_adb, base->trackpad_mode);
-	} else if (base->flags.set_kbdmode == 1) {
+	if (base->flags.set_kbdmode == 1) {
 		base->flags.set_kbdmode = 0;
 		keyboard_set_config(base->fd_adb, base->keyboard_mode);
 	} else if (base->flags.goto_sleep == 1) {
@@ -278,29 +364,29 @@ pmac_configure (struct tagitem *taglist)
 	pmac_handle_tags (MODE_CONFIG, taglist);
 }
 
-int
+void
 pmac_handle_tags (int cfgure, struct tagitem *taglist)
 {
 	struct moddata_pmac *base = &modbase_pmac;
-	int err, rc = 0;
+	int err;
 
 	while (taglist->tag != TAG_END) {
 		switch (taglist->tag) {
 		case TAG_REINIT:      /* system tag */
-			base->flags.set_tpmode = 1;     /* reset trackpad mode */
+			trackpad_set_config(base->fd_adb, base->trackpad_mode);
 			base->flags.set_kbdmode = 1;     /* reset keyboard mode */
 			break;
 		case TAG_PMUDEVICE:
 			if (cfgure) {
 				if ((err = copy_path ((char *) taglist->data, base->pmudev, TYPE_CHARDEV, CPFLG_NONE)))
-					rc = tagerror (taglist, err);
+					tagerror (taglist, err);
 			} else
 				taglist->data = (long) base->pmudev;
 			break;
 		case TAG_ADBDEVICE:
 			if (cfgure) {
 				if ((err = copy_path ((char *) taglist->data, base->adbdev, TYPE_CHARDEV, CPFLG_NONE)))
-					rc = tagerror (taglist, err);
+					tagerror (taglist, err);
 			} else
 				taglist->data = (long) base->adbdev;
 			break;
@@ -331,11 +417,15 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 		case TAG_TPMODE:
 			if (cfgure)	{
 				base->trackpad_mode = taglist->data > TRACKPAD_LAST ? 0 : taglist->data;
-				base->flags.set_tpmode = 1;     /* request a secure function call */
+				trackpad_set_config(base->fd_adb, base->trackpad_mode);
 				singletag_to_clients (CHANGEVALUE, TAG_TPMODE, base->trackpad_mode);
 			} else
 				taglist->data = base->trackpad_mode;
 			break;
+		case TAG_NOTAPTYPING:
+			if (cfgure)	base->flags.notap_typing = taglist->data & 1;
+			else		taglist->data = base->flags.notap_typing;
+			break;
 		case TAG_KBDMODE:
 			if (cfgure) {
 				if (base->version != OHARE_PMU) {
@@ -411,23 +501,39 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 		}
 		taglist++;
 	}
-	return rc;
 }
 
-void
+int
 pmac_pmu_handler (int fd)
 {
 	unsigned char intr[16];
+	struct tagitem taglist[4];
 	int n;
+	static int PBpressed = 0;
 #if defined(DEBUG) && PMUINTR
 	int i;
 #endif
 	if ((n = read(fd, intr, sizeof(intr))) > 0) {
-		switch(intr[0]) {
+		switch (intr[0]) {
 		case 4:
 			if (n < 3 || intr[2] < 1 || intr[2] > 2)
 				break;
 			call_script ("/sbin/cardctl %s %d", "eject", intr[2]-1);
+			break;
+		case 0x40:  /* timer interrupt */
+			/* no battery present -> no timer interrupts */
+			/* n = 2 && intr[1] = 0x14 = %10100 if a battery is present */
+			/* n = 6 && intr[1] = 0x1c = %11100 if power button is pressed */
+			/* n = 6 && intr[1] = 0x15 = %10101 after cover close */
+			if (n == 6 && ((intr[1] >> 3) & 1) != PBpressed) {
+				PBpressed = (intr[1] >> 3) & 1;
+				taglist_init (taglist);
+				taglist_add (taglist, TAG_KEYCODE, KEY_POWER);
+				taglist_add (taglist, TAG_KEYREPEAT, PBpressed ? 1 : 0);
+				taglist_add (taglist, TAG_MODIFIER, 0);
+				process_queue (KBDQUEUE, taglist);
+			}
+			break;
 		}
 #if defined(DEBUG) && PMUINTR
 		printf ("\nPMU interrupt: ");
@@ -436,6 +542,8 @@ pmac_pmu_handler (int fd)
 		printf ("\n");
 #endif
 	}
+	return 0; /* we assume that the filehandle for /dev/pmu can't
+				 be closed from outside pbbuttonsd */
 }
 
 void
@@ -484,7 +592,20 @@ pmac_timer (struct tagitem *taglist)
 			}
 		}
 	}
-
+	
+#if 0
+	/* trackpad tapping control */
+	if (base->trackpad_pause) {
+		if ((--base->trackpad_pause) == 0) {
+			trackpad_set_config(base->fd_adb, base->trackpad_mode);
+#if defined(DEBUG) && TRACKPAD
+			printf ("Trackpad tapping enabled \r");
+			fflush (stdout);
+#endif	
+		}
+	}
+#endif
+	
 	if (args[0].tag != TAG_END)
 		process_queue (CONFIGQUEUE, args);  /* distribute changes to other modules */
 	base->flagschanged = 0;
@@ -495,37 +616,45 @@ pmac_timer (struct tagitem *taglist)
 
 /* This function put the machine into sleep mode. It tries pmud first. If it was not
    available, it would go directly over the PMU.
+   
    Very importent is, that no modifier key is pressed, when this function is called,
    because this would confuse the input driver after wakeup. This case is not
    fully avoided, because this function is not atomic and with some fast actions
-   the user could bypass our security gate. */
-
+   the user could bypass our security gate.
+   
+   The second point is that current states of the trackpad and the keyboard will
+   only stored locally. This will normally have no side effects but the global
+   variables base->trackpad_mode and base->keyboard-mode won't be synchronized
+   with the hardware here anymore. So the states set by external programs will
+   survive sleep but also will be overwritten as soon as pbbuttonsd access one
+   of the devices.
+*/
 void
 activate_sleepmode ()
 {
 	struct moddata_pmac *base = &modbase_pmac;
+	int tpmode, kbdmode = 0; 
 #ifdef WITH_PMUD
 	char buf[35];
 	int fd;
 #endif
 
-	base->trackpad_mode = trackpad_get_config (base->fd_adb);
+	tpmode = trackpad_get_config (base->fd_adb);
 	if (base->version != OHARE_PMU)
-		base->keyboard_mode = keyboard_get_config (base->fd_adb);
+		kbdmode = keyboard_get_config (base->fd_adb);
 	sync();
-
 #ifndef WITH_PMUD
 	/* PMU_IOC_SLEEP only succeeds if called as root.
 	 * It returns immediatly if not, without putting
 	 * the mashine to sleep
 	 */
 	ioctl (base->fd_pmu, PMU_IOC_SLEEP, 0);    /* returns after next awake */
-	trackpad_set_config(base->fd_adb, base->trackpad_mode);
+	trackpad_set_config(base->fd_adb, tpmode);
 #else
 	if (((fd = tcp_open("localhost", "pmud", 879)) < 0)) {
-		print_error(_("WARNING: Can't find pmud on port 879. Trigger PMU directly for sleep.\n"));
+		print_msg (PBB_WARN, _("Can't find pmud on port 879. Trigger PMU directly for sleep.\n"));
 		ioctl (base->fd_pmu, PMU_IOC_SLEEP, 0);     /* returns after next awake */
-		trackpad_set_config(base->fd_adb, base->trackpad_mode);
+		trackpad_set_config(base->fd_adb, tpmode);
 	} else {
 		tcp_printf(fd, "sleep\n");
 
@@ -539,7 +668,7 @@ activate_sleepmode ()
 	}
 #endif
 	if (base->version != OHARE_PMU)
-		keyboard_set_config(base->fd_adb, base->keyboard_mode);
+		keyboard_set_config(base->fd_adb, kbdmode);
 }
 
 /* This function gets the battery status from the PMU kernel driver via
@@ -679,6 +808,35 @@ pmac_update_flags()
 	return (envold ^ envnew);
 }
 
+int
+pmac_getmachine ()
+{
+	char buffer[100], *token;
+	int machine = 0, n;
+	FILE *fd;
+
+	if ((fd = fopen ("/proc/cpuinfo", "r"))) {
+		while (fgets (buffer,sizeof (buffer), fd)) {
+			cleanup_buffer (buffer);
+			if ((token = strtok (buffer, ":\n"))) {
+				if (strncmp ("machine", token, 7))
+					continue;
+				
+				token = strtok (0, "\n");
+				if (!strncmp ("PowerBook", token, 9)) {
+					machine = (atoi (&token[9]) & 0xf) << 4;
+					for (n=9; token[n] != ',' && token[n] != '\n'; n++);
+					if (token[n] == ',')
+						machine |= (atoi (&token[n+1]) & 0xf);
+				}
+				break;
+			}
+		}
+		fclose(fd);
+	}
+	return machine;
+}
+
 int
 pmac_get_lcdbacklight ()
 {
@@ -733,6 +891,69 @@ pmac_identify (char* buffer, int pmu)
   snprintf(buffer, STDBUFFERLEN, "%s (PMU version: %d)", version[idx], pmu);
 }
 
+int
+isModifier (int code, int mod)
+{
+	if (mod) {
+		switch (code) {
+			case KEY_LEFTCTRL:
+			case KEY_LEFTALT:
+			case KEY_LEFTSHIFT:
+			case KEY_LEFTMETA:
+			case KEY_RIGHTCTRL:
+			case KEY_RIGHTALT:
+			case KEY_RIGHTSHIFT:
+			case KEY_RIGHTMETA:
+			case KEY_COMPOSE:
+			case KEY_CAPSLOCK:
+			case KEY_NUMLOCK:
+			case KEY_SCROLLLOCK:
+				return 1;  /* key is modifier */
+		}
+	}
+	return 0;  /* key is not a modifier */
+}
+
+int
+haveADBKeyboard ()
+{
+	struct moddata_pmac *base = &modbase_pmac;
+	
+	if (base->machine > 0) {
+		if (base->machine < 0x56)              /*  0 - 56 -> true  */
+			return 1;
+		if (base->machine > 0x68)              /* 69 - ?? -> false */
+			return 0;
+		if (((base->machine >> 4) & 0xf) == 5)
+			if (base->machine >= 0x56)         /* 56 - 5f -> false */
+				return 0;
+		if (((base->machine >> 4) & 0xf) == 6)
+			if (base->machine <= 0x68)         /* 60 - 68 -> true  */
+				return 1;
+	}
+	return 0;
+}
+
+int
+haveADBTrackpad ()
+{
+	struct moddata_pmac *base = &modbase_pmac;
+	
+	if (base->machine > 0) {
+		if (base->machine < 0x56)              /*  0 - 56 -> true  */
+			return 1;
+		if (base->machine > 0x67)              /* 68 - ?? -> false */
+			return 0;
+		if (((base->machine >> 4) & 0xf) == 5)
+			if (base->machine >= 0x56)         /* 56 - 5f -> false */
+				return 0;
+		if (((base->machine >> 4) & 0xf) == 6)
+			if (base->machine <= 0x67)         /* 60 - 67 -> true  */
+				return 1;
+	}
+	return 0;
+}
+
 /* ------------------  POWER MAC SPECIFIC KEYBOARD FUNCTIONS ------------------- */
 
 int
@@ -741,11 +962,13 @@ keyboard_get_config (int fd)
 	unsigned char ADBBuffer[ADB_BUFSIZE];
 	int n, dev, config = -1;
 
-	if ((dev = keyboard_get_dev (fd)) > 0) {
-		n = adb_read_reg (fd, ADBBuffer, dev, 1);
-		if (n > 0) {
-			config = 0;
-			config |= ADBBuffer[3] & 0x01 ? KEYBOARD_FNTOP : 0;
+	if (haveADBKeyboard()) {
+		if ((dev = keyboard_get_dev (fd)) > 0) {
+			n = adb_read_reg (fd, ADBBuffer, dev, 1);
+			if (n > 0) {
+				config = 0;
+				config |= ADBBuffer[3] & 0x01 ? KEYBOARD_FNTOP : 0;
+			}
 		}
 	}
 	return config;
@@ -757,12 +980,14 @@ keyboard_set_config (int fd, int config)
 	unsigned char ADBBuffer[ADB_BUFSIZE];
 	int n, dev;
 
-	if ((dev = keyboard_get_dev (fd)) > 0) {
-		n = adb_read_reg (fd, ADBBuffer, dev, 1);
-		if (n > 0) {
-			ADBBuffer[3] &= 0xfe;
-			ADBBuffer[3] |= (config & KEYBOARD_FNTOP) ? 0x01 : 0x00;
-			adb_write_reg (fd, ADBBuffer, n, dev, 1);
+	if (haveADBKeyboard()) {
+		if ((dev = keyboard_get_dev (fd)) > 0) {
+			n = adb_read_reg (fd, ADBBuffer, dev, 1);
+			if (n > 0) {
+				ADBBuffer[3] &= 0xfe;
+				ADBBuffer[3] |= (config & KEYBOARD_FNTOP) ? 0x01 : 0x00;
+				adb_write_reg (fd, ADBBuffer, n, dev, 1);
+			}
 		}
 	}
 }
@@ -792,20 +1017,22 @@ trackpad_get_config (int fd)
 	unsigned char ADBBuffer[ADB_BUFSIZE];
 	int n, dev, config = -1;
 
-	if ((dev = trackpad_get_dev (fd)) > 0) {
-		trackpad_set_prgmode (fd, dev, 1);
-		n = adb_read_reg (fd, ADBBuffer, dev, 2);
-		if (n > 0) {
-			if (ADBBuffer[5] == 0xff)
-				config = TRACKPAD_LOCK;
-			else if (ADBBuffer[3] == 0x94)
-				config = TRACKPAD_DRAG;
-			else if (ADBBuffer[2] == 0x99)
-				config = TRACKPAD_TAP;
-			else
-				config = TRACKPAD_NOTAP;
+	if (haveADBTrackpad()) {
+		if ((dev = trackpad_get_dev (fd)) > 0) {
+			trackpad_set_prgmode (fd, dev, 1);
+			n = adb_read_reg (fd, ADBBuffer, dev, 2);
+			if (n > 0) {
+				if (ADBBuffer[5] == 0xff)
+					config = TRACKPAD_LOCK;
+				else if (ADBBuffer[3] == 0x94)
+					config = TRACKPAD_DRAG;
+				else if (ADBBuffer[2] == 0x99)
+					config = TRACKPAD_TAP;
+				else
+					config = TRACKPAD_NOTAP;
+			}
+			trackpad_set_prgmode (fd, dev, 0);
 		}
-		trackpad_set_prgmode (fd, dev, 0);
 	}
 	return config;
 }
@@ -816,16 +1043,18 @@ trackpad_set_config (int fd, int config)
 	unsigned char ADBBuffer[ADB_BUFSIZE];
 	int n, dev;
 
-	if ((dev = trackpad_get_dev (fd)) > 0) {
-		trackpad_set_prgmode (fd, dev, 1);
-		n = adb_read_reg (fd, ADBBuffer, dev, 2);
-		if (n > 0) {
-			ADBBuffer[2] = (config < TRACKPAD_TAP) ? 0x19 : 0x99;
-			ADBBuffer[3] = (config < TRACKPAD_DRAG) ? 0x14 : 0x94;
-			ADBBuffer[5] = (config < TRACKPAD_LOCK) ? 0xb2 : 0xff;
-			adb_write_reg (fd, ADBBuffer, n, dev, 2);
+	if (haveADBTrackpad()) {
+		if ((dev = trackpad_get_dev (fd)) > 0) {
+			trackpad_set_prgmode (fd, dev, 1);
+			n = adb_read_reg (fd, ADBBuffer, dev, 2);
+			if (n > 0) {
+				ADBBuffer[2] = (config < TRACKPAD_TAP) ? 0x19 : 0x99;
+				ADBBuffer[3] = (config < TRACKPAD_DRAG) ? 0x14 : 0x94;
+				ADBBuffer[5] = (config < TRACKPAD_LOCK) ? 0xb2 : 0xff;
+				adb_write_reg (fd, ADBBuffer, n, dev, 2);
+			}
+			trackpad_set_prgmode (fd, dev, 0);
 		}
-		trackpad_set_prgmode (fd, dev, 0);
 	}
 }
 
@@ -968,11 +1197,10 @@ get_int_environment(int fd)
 
 /* ----------------------------- LOW LEVEL LMU FUNCTIONS ---------------------------- */
 
-/* This function returns the a combined and compensated ambient
- * light sensor value in percent. The reduce flickering through
- * shadows or spot lights at one of the sensors each sensor is
- * weighted only half of full scale. In result spikes from one
- * sensor will be damped.
+/* This function returns the a compensated ambient light sensor
+ * value in percent. The reduce flickering through shadows or
+ * spot lights at one of the sensors the sensor with the highest
+ * value is choosen.
  */
 int
 lmu_get_ambient ()
@@ -981,9 +1209,9 @@ lmu_get_ambient ()
 	int rc;
 
 	rc = (int) (((lmu_get_raw_ambient(base->i2c_bus_lmu) - base->lcdfeedback) 
-				* 50) / AMBIENTMAX);
-	if (rc < 0)	rc = 0;
-	if (rc > 100)	rc = 100;
+				* 100) / AMBIENTMAX);
+	if (rc < 0)	  rc = 0;
+	if (rc > 100) rc = 100;
 	return rc;
 }
 
@@ -1010,7 +1238,7 @@ lmu_find_bus ()
 			if (ioctl (fd, I2C_SLAVE, LMU_ADDR) >= 0 ) {
 				if (read (fd, buf, 4) == 4 ) {
 					close (fd);
-					rc = n;
+					rc = (n & 255);
 					break;
 				}
 			}
@@ -1058,6 +1286,8 @@ lmu_set_illumination (int dev, unsigned short level)
 /* This function reads the ambient light sensor. New aluminum
  * PowerBooks have two of them left and right below the speaker
  * covers.
+ * This function asks both sensors and the highest value is
+ * returned
  */
 int
 lmu_get_raw_ambient (int dev)
@@ -1065,19 +1295,24 @@ lmu_get_raw_ambient (int dev)
 #if defined(DEBUG) && SIMUAMBIENT
 	struct moddata_pmac *base = &modbase_pmac;
 	int ambient, level;
-	int asens[] = {2+2, 182+135, 217+167, 248+200, 309+281,
-	               337+295, 381+341, 423+391, 461+426, 499+473,
-	               538+515, 576+563, 620+669,
-		       1072+1072, 1073+1076, 1077+1080};
+
+	/* this array contains sensor values depending on the
+	 * LCD brightness without additional ambient light.
+	 * They simulate the LCD brightness influence on the
+	 * light sensors and are for debugging only.
+	 */
+	int asens[] = {2, 182, 217, 248, 309, 337, 381, 423, 461,
+		           499, 538, 576, 620, 1072, 1073, 1077};
 		       
 	level = pmac_get_lcdbacklight();
-	ambient = asens[level == -1 ? 0 : level] + base->ambient * 2;
-	if (ambient > 2*AMBIENTMAX)
-		ambient = 2*AMBIENTMAX;
+	ambient = asens[level == -1 ? 0 : level] + base->ambient;
+	if (ambient > AMBIENTMAX)
+		ambient = AMBIENTMAX;
+	
 	return ambient;
 #else
 	char i2cdevice[20], buf[4];
-	int fd, rc = -1;
+	int fd, rc = -1, rc2;
 
 	if (dev >= 0) {
 		snprintf(i2cdevice, 19, "/dev/i2c-%d", dev );
@@ -1085,7 +1320,8 @@ lmu_get_raw_ambient (int dev)
 			if (ioctl (fd, I2C_SLAVE, LMU_ADDR) >= 0 )
 				if (read (fd, buf, 4 ) == 4) {
 					rc  = (buf[0] << 8) | buf[1];
-					rc += (buf[2] << 8) | buf[3];
+					rc2 = (buf[2] << 8) | buf[3];
+					rc  = rc > rc2 ? rc : rc2; 
 				}
 			close (fd);
 		}
@@ -1124,7 +1360,7 @@ batlog_setup()
 			}
 		}
 		snprintf (batlog, STDBUFFERLEN, DEFAULT_BATLOG, cycle);
-		print_error(_("INFO: Current battery cycle: %d, active logfile: %s.\n"),
+		print_msg (PBB_INFO, _("Current battery cycle: %d, active logfile: %s.\n"),
 			cycle, base->batlog_mode == BATLOG_LOG ? batlog : "none");
 	}
 	return cycle;
