@@ -34,6 +34,7 @@
 
 #include "gettext_macros.h"
 #include "input_manager.h"
+#include "config_manager.h"
 #include "module_powersave.h"
 #include "module_display.h"
 #include "support.h"
@@ -76,24 +77,108 @@ struct moddata_power {
    at the needed input queues. */
 
 int
-power_init (struct tagitem *taglist)
+power_init ()
 {
 	struct moddata_power *base = &modbase_power;
 	static char scriptbuffer1[STDBUFFERLEN];
 	static char scriptbuffer2[STDBUFFERLEN];
-	int rc;
+	int sid;
 
-	power_sync(); /* syncronise some data with module_pmac */
+	bzero (base, sizeof (struct moddata_power));
+	bzero (scriptbuffer1, STDBUFFERLEN);  /* no default for script_pmcs */
+	sprintf (scriptbuffer2, DEFAULT_NETLOAD_DEV);
+	base->script_pmcs  = scriptbuffer1;
+	base->netload_dev  = scriptbuffer2;
+	base->mode         = MODE_AWAKE;
+	base->policy       = POLICY_PERFORMANCE;
+	
+	if ((sid = registerCfgSection ("MODULE POWERSAVE")) == 0) {
+		print_msg (PBB_ERR, _("Can't register configuration section %s, out of memory."), "MODULE POWERSAVE");
+		return E_NOMEM;
+	} else {
+		registerCfgOptionList (sid, "onAC_Policy", TAG_ONAC_POLICY, 0,
+				N_("'"POLICY_NONE_NAME"', '"POLICY_POWERSAVE_NAME"', '"POLICY_USER_NAME"' or '"POLICY_PERFORMANCE_NAME"'"),
+				POLICY_NONE_NAME, POLICY_POWERSAVE_NAME, POLICY_USER_NAME, POLICY_PERFORMANCE_NAME, NULL);
+		registerCfgOptionList (sid, "onAC_TimerAction", TAG_ONAC_TIMERACTION, 0,
+				N_("'"ACTION_NONE_NAME"', '"ACTION_TORAM_NAME"', '"ACTION_TODISK_NAME"' or '"ACTION_BLANK_NAME"'"),
+				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, NULL);
+		registerCfgOptionList (sid, "onAC_CoverAction",  TAG_ONAC_COVERACTION, 0,
+				N_("see TimerAction for possible values"),
+				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, NULL);
+		registerCfgOptionList (sid, "onAC_KeyAction", TAG_ONAC_KEYACTION, 0,
+				N_("Action (see TimerAction) for the sleepkey"),
+				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, NULL);
+		registerCfgOptionInt (sid, "onAC_SuspendTime", TAG_ONAC_TIMESUSPEND, 0,
+				NULL);
+		registerCfgOptionInt (sid, "onAC_DimTime",     TAG_ONAC_TIMEDIM,     0,
+				NULL);
 
-	scriptbuffer1[0]            = 0; /* no default for script_pmcs */
-	base->script_pmcs           = scriptbuffer1;
-	base->mode                  = MODE_AWAKE;
+		registerCfgOptionList (sid, "onBattery_Policy", TAG_ONBATT_POLICY, 0,
+				N_("'"POLICY_NONE_NAME"', '"POLICY_POWERSAVE_NAME"', '"POLICY_USER_NAME"' or '"POLICY_PERFORMANCE_NAME"'"),
+				POLICY_NONE_NAME, POLICY_POWERSAVE_NAME, POLICY_USER_NAME, POLICY_PERFORMANCE_NAME, NULL);
+		registerCfgOptionList (sid, "onBattery_TimerAction", TAG_ONBATT_TIMERACTION, 0,
+				N_("'"ACTION_NONE_NAME"', '"ACTION_TORAM_NAME"', '"ACTION_TODISK_NAME"' or '"ACTION_BLANK_NAME"'"),
+				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, NULL);
+		registerCfgOptionList (sid, "onBattery_CoverAction",  TAG_ONBATT_COVERACTION, 0,
+				N_("see TimerAction for possible values"),
+				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, NULL);
+		registerCfgOptionList (sid, "onBattery_KeyAction", TAG_ONBATT_KEYACTION, 0,
+				N_("Action (see TimerAction) for the sleepkey"),
+				ACTION_NONE_NAME, ACTION_TORAM_NAME, ACTION_TODISK_NAME, ACTION_BLANK_NAME, NULL);
+		registerCfgOptionInt (sid, "onBattery_SuspendTime", TAG_ONBATT_TIMESUSPEND, 0,
+				NULL);
+		registerCfgOptionInt (sid, "onBattery_DimTime",     TAG_ONBATT_TIMEDIM,     0,
+				NULL);
+		
+		registerCfgOptionKey (sid, "SleepKey", TAG_SLEEPKEY, TAG_SLEEPMOD, 0,
+				NULL);
+		registerCfgOptionInt (sid, "SleepKeyDelay", TAG_SLEEPKEYDELAY, 0,
+				N_("values > 0 may be dangerous, if the power key is used to trigger sleep"));
+		registerCfgOptionInt (sid, "BWL_first", TAG_BWLFIRST, 0,
+				N_("first battery warnlevel, time in minutes"));
+		registerCfgOptionInt (sid, "BWL_second", TAG_BWLSECOND, 0,
+				N_("second battery warnlevel, time in minutes"));
+		registerCfgOptionInt (sid, "BWL_last", TAG_BWLLAST, 0,
+				N_("last battery warnlevel, time in minutes"));
+		registerCfgOptionString (sid, "Script_PMCS", TAG_SCRIPTPMCS, 0,
+				NULL);
+		registerCfgOptionList (sid, "EmergencyAction", TAG_EMERGENCYACTION, 0,
+				N_("action, if battery is critically low"),
+				EMA_SLEEP_NAME, EMA_SIGNAL_NAME, EMA_COMMAND_NAME, NULL);
+		registerCfgOptionBool (sid, "HeartbeatBeep", TAG_HEARTBEATBEEP, 0,
+				N_("beep, if nothing else showed that the computer lives"));
+		registerCfgOptionBool (sid, "CPULoad_sleeplock", TAG_CPULOADSLEEPLOCK, 0,
+				NULL);
+		registerCfgOptionInt (sid, "CPULoad_min", TAG_CPULOADMIN, 0,
+				N_("value in percent"));
+		registerCfgOptionInt (sid, "CPULoad_period", TAG_CPULOADPERIOD, 0,
+				N_("time in seconds"));
+		registerCfgOptionBool (sid, "NETLoad_sleeplock", TAG_NETLOADSLEEPLOCK, 0,
+				NULL);
+		registerCfgOptionInt (sid, "NETLoad_min", TAG_NETLOADMIN, 0,
+				N_("traffic in Bytes/s"));
+		registerCfgOptionInt (sid, "NETLoad_period", TAG_NETLOADPERIOD, 0,
+				N_("time in seconds"));
+		registerCfgOptionString (sid, "NETLoad_device", TAG_NETLOADDEV, 0,
+				NULL);
+	}
 
-	base->policy                = POLICY_PERFORMANCE;
-	base->flags.usersetpolicy   = 0;
+	ipc_protect_tag (TAG_SCRIPTPMCS);  /* tags only for privileged use */
+
+	register_function (QUERYQUEUE, power_query);
+	register_function (CONFIGQUEUE, power_configure);
+	return 0;
+}
+
+int
+power_open (struct tagitem *taglist)
+{
+	struct moddata_power *base = &modbase_power;
+	
+	power_sync(); /* synchronise some data with module_pmac */
 	
 	/* default profile for running on AC power */
-	base->onAC.policy	    = POLICY_PERFORMANCE;
+	base->onAC.policy	        = POLICY_PERFORMANCE;
 	base->onAC.timeraction      = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
 	base->onAC.coveraction      = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
 	base->onAC.keyaction        = base->flags.sleep_supported ? ACTION_TORAM : ACTION_BLANK;
@@ -132,31 +217,27 @@ power_init (struct tagitem *taglist)
 	base->flags.sleeplock_net   = 0;    /* lock open */
 	base->netload_min           = 4096; /* lock closed if netload <4 KB/s */
 	base->netload_period        = 60;   /*  for >60s, if enabled  */
-	sprintf(scriptbuffer2, DEFAULT_NETLOAD_DEV);
-	base->netload_dev           = scriptbuffer2;
 
 	base->flags.heartbeat       = 0;  /* heartbeat beep disabled */
 	base->flags.heartbeat_enable= 1;  /* heartbeat beep is possible by default */ 
 	base->flags.coveropen       = 1;  /* cover open by default same as in laptop module */
 
-	if ((rc = power_handle_tags (MODE_CONFIG, taglist))) /* interprete configfile */
-		return rc;         /* critcal error occured, abort program */
-
 	base->powersource = 2; /* HACK: The pmcs script sould be started at startup but
 				  it will only start if the status has changed (policy
-				  or powersource). We set an impossible value here after
-				  the module is fully initialized and pbbuttonsd.c will
-				  send a TAG_POWERCHANGE to correct it and start the script
+				  or powersource). We set an impossible value here and pbbuttonsd.c
+				  will send a TAG_POWERCHANGE to correct it and start the script
 				  Not elegant but works */
 	
-	ipc_protect_tag (TAG_SCRIPTPMCS);  /* tags only for privileged use */
-
 	register_function (KBDQUEUE, power_keyboard);
 	register_function (MOUSEQUEUE, power_mouse);
 	register_function (T100QUEUE, power_timer);
 	register_function (T1000QUEUE, power_timer1000);
-	register_function (QUERYQUEUE, power_query);
-	register_function (CONFIGQUEUE, power_configure);
+	return 0;
+}
+
+int
+power_close ()
+{
 	return 0;
 }
 
@@ -244,11 +325,11 @@ power_configure (struct tagitem *taglist)
   power_handle_tags (MODE_CONFIG, taglist);
 }
 
-int
+void
 power_handle_tags (int cfgure, struct tagitem *taglist)
 {
 	struct moddata_power *base = &modbase_power;
-	int err, rc = 0, val;
+	int err, val;
 
 	while (taglist->tag != TAG_END) {
 		switch (taglist->tag) {
@@ -261,10 +342,10 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 			if (cfgure) {
 				if ((check_script ((char *) taglist->data, "sss")) == 0) {
 					if ((err = copy_path ((char *) taglist->data, base->script_pmcs, TYPE_FILE, CPFLG_PBBONLY)))
-						rc = tagerror (taglist, err);
+						tagerror (taglist, err);
 				} else {
-					print_error(_("ERROR: Too many formatsigns. Max three %%s allowed in TAG_SCRIPTPMCS.\n"));
-					rc = tagerror (taglist, E_FORMAT);
+					print_msg (PBB_ERR, _("Too many formatsigns. Max three %%s allowed in TAG_SCRIPTPMCS.\n"));
+					tagerror (taglist, E_FORMAT);
 				}
 			} else
 				taglist->data = (long) base->script_pmcs;
@@ -272,7 +353,6 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 			
 		/* Tags for power profiles */	
 		case TAG_ONAC_POLICY:
- 		case TAG_ONACPOLICY:      /* depreceated */
 			if (cfgure) {
 				if (taglist->data > POLICY_LAST)
 					tagerror(taglist, E_INVALID);
@@ -313,26 +393,15 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 				taglist->data = base->onAC.keyaction;
 			break;	
 		case TAG_ONAC_TIMESUSPEND:
-		case TAG_ONACTIMESLEEP:   /* depreceated */
 			if (cfgure)	base->onAC.suspendtime = taglist->data;
 			else		taglist->data = base->onAC.suspendtime;
 			break;
 		case TAG_ONAC_TIMEDIM:
-		case TAG_ONACTIMEDIM:     /* depreciated */
 			if (cfgure)	base->onAC.dimtime = taglist->data;
 			else		taglist->data = base->onAC.dimtime;
  			break;
-		case TAG_ONACSLEEP:       /* depreceated */
-			if (cfgure)	base->onAC.timeraction = taglist->data & 1 ? ACTION_TORAM : ACTION_BLANK;
-			else 		taglist->data = (base->onAC.timeraction == ACTION_TORAM || base->onAC.timeraction == ACTION_TODISK) ? 1 : 0;
-			break;
-		case TAG_ONACCOVERSLEEP:  /* depreceated */
-			if (cfgure)	base->onAC.coveraction = taglist->data & 1 ? ACTION_TORAM : ACTION_BLANK;
-			else 		taglist->data = (base->onAC.coveraction == ACTION_TORAM || base->onAC.coveraction == ACTION_TODISK) ? 1 : 0;
-			break;
 	
 		case TAG_ONBATT_POLICY:	
-		case TAG_ONBATTERYPOLICY: /* depreciated */
 			if (cfgure) {
 				if (taglist->data > POLICY_LAST)
 					tagerror(taglist, E_INVALID);
@@ -373,23 +442,13 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 				taglist->data = base->onBattery.keyaction;
 			break;
 		case TAG_ONBATT_TIMESUSPEND:
-		case TAG_ONBATTERYTIMESLEEP:  /* depreciated */
 			if (cfgure)	base->onBattery.suspendtime = taglist->data;
 			else		taglist->data = base->onBattery.suspendtime;
 			break;
 		case TAG_ONBATT_TIMEDIM:
-		case TAG_ONBATTERYTIMEDIM:    /* depreceated */
 			if (cfgure)	base->onBattery.dimtime = taglist->data;
 			else		taglist->data = base->onBattery.dimtime;
 			break;
-		case TAG_ONBATTERYSLEEP:      /* depreceated */
-			if (cfgure)	base->onBattery.timeraction = taglist->data & 1 ? ACTION_TORAM : ACTION_BLANK;
-			else 		taglist->data = (base->onBattery.timeraction == ACTION_TORAM || base->onBattery.timeraction == ACTION_TODISK) ? 1 : 0;
-			break;
-		case TAG_ONBATTERYCOVERSLEEP: /* depreceated */
-			if (cfgure)	base->onBattery.coveraction = taglist->data & 1 ? ACTION_TORAM : ACTION_BLANK;
-			else 		taglist->data = (base->onBattery.coveraction == ACTION_TORAM || base->onBattery.coveraction == ACTION_TODISK) ? 1 : 0;
-			break;
 		/* Tags for power profiles end */
 
 		case TAG_SLEEPKEY:
@@ -404,13 +463,6 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 			if (cfgure)	base->sleepkeydelaytime = taglist->data;
 			else		taglist->data = base->sleepkeydelaytime;
 			break;
-		case TAG_SLEEPKEYSLEEP:  /* depreceated */
-			if (cfgure) {
-				base->onAC.keyaction = taglist->data & 1 ? ACTION_TORAM : ACTION_BLANK;
-				base->onBattery.keyaction = taglist->data & 1 ? ACTION_TORAM : ACTION_BLANK;
-			} else 	
-				taglist->data = (base->onAC.keyaction == ACTION_TORAM || base->onAC.keyaction == ACTION_TODISK) ? 1 : 0;
-			break;
 		case TAG_BWLFIRST:
 			if (cfgure)	base->BWL1 = taglist->data;
 			else		taglist->data = base->BWL1;
@@ -488,12 +540,11 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 				power_setpolicy(base->activeProfile->policy, taglist->data);
 				base->flags.usersetpolicy = 0;
 				
-				/* Because the timeout values for sleeping and
-				 * dimming have been changed the sleeptime counter
-				 * have to be adjusted to prevent unexpected jumps
-				 * in behaviour. Depending on the current mode the
-				 * sleeptime counter will be resetted to the begining
-				 * of the current mode.
+				/* Due to changed timeout values of sleeping and
+				 * dimming the sleeptime counter have to be adjusted
+				 * to prevent unexpected jumps in behaviour. Depending
+				 * on the current mode the sleeptime counter will be
+				 * resetted to the begining of the current mode.
 				 */
 				if (base->mode == MODE_AWAKE)
 					base->sleeptime = 0;
@@ -536,7 +587,8 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 				if (base->flags.coveropen) 
  					power_awake ();
 				else {
-					power_suspend (ACTION_BLANK); /* always blank screen if cover has been closed */
+					if (val != ACTION_BLANK)
+						power_suspend (ACTION_BLANK); /* always blank screen if cover has been closed */
 					power_suspend (val);          /* then follow user's will */
 				}
 			}
@@ -559,7 +611,6 @@ power_handle_tags (int cfgure, struct tagitem *taglist)
 		}
 		taglist++;
 	}
-	return rc;
 }
 
 /* This routine will be called every 10ms */
