@@ -4,7 +4,7 @@
  * module_powersave.h
  * prototypes and definitions for the module powermanagement.
  *
- * Copyright 2002 Matthias Grimm
+ * Copyright 2002-2005 Matthias Grimm
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -94,7 +94,9 @@ struct powerprofile {
 };
 
 /* public prototypes */
-int power_init (struct tagitem *taglist);
+int power_init ();
+int power_open (struct tagitem *taglist);
+int power_close ();
 int power_exit ();
 
 /* private prototypes */
@@ -104,7 +106,7 @@ void power_awake ();
 void power_sync ();
 void power_query (struct tagitem *taglist);
 void power_configure (struct tagitem *taglist);
-int power_handle_tags (int cfgure, struct tagitem *taglist);
+void power_handle_tags (int cfgure, struct tagitem *taglist);
 void power_timer (struct tagitem *taglist);
 void power_timer1000 (struct tagitem *taglist);
 void power_check_battery (int time, int psource);
